%%% radius.hrl
%%%

%% RADIUS Codes
-define(AccessRequest,       1).
-define(AccessAccept,        2).
-define(AccessReject,        3).
-define(AccountingRequest,   4).
-define(AccountingResponse,  5).
-define(AccessChallenge,    11).
-define(StatusServer,       12).
-define(StatusClient,       13).
-define(DisconnectRequest,  40).
-define(DisconnectAck,      41).
-define(DisconnectNak,      42).
-define(CoARequest,         43).
-define(CoAAck,             44).
-define(CoANak,             45).

-record(radius,
      {code :: byte(),
      id :: byte(),
      authenticator :: binary() | [byte()],
      attributes :: binary()
				| radius_attributes:attributes()}).

%% RADIUS Attribute Types
-define(UserName,                          1).
-define(UserPassword,                      2).
-define(ChapPassword,                      3).
-define(NasIpAddress,                      4).
-define(NasPort,                           5).
-define(ServiceType,                       6).
-define(FramedProtocol,                    7).
-define(FramedIpAddress,                   8).
-define(FramedIpNetmask,                   9).
-define(FramedRouting,                    10).
-define(FilterId,                         11).
-define(FramedMtu,                        12).
-define(FramedCompression,                13).
-define(LoginIpHost,                      14).
-define(LoginService,                     15).
-define(LoginTcpPort,                     16).
-define(ReplyMessage,                     18).
-define(CallbackNumber,                   19).
-define(CallbackId,                       20).
-define(FramedRoute,                      22).
-define(FramedIpxNetwork,                 23).
-define(State,                            24).
-define(Class,                            25).
-define(VendorSpecific,                   26).
-define(SessionTimeout,                   27).
-define(IdleTimeout,                      28).
-define(TerminationAction,                29).
-define(CalledStationId,                  30).
-define(CallingStationId,                 31).
-define(NasIdentifier,                    32).
-define(ProxyState,                       33).
-define(LoginLatService,                  34).
-define(LoginLatNode,                     35).
-define(LoginLatGroup,                    36).
-define(FramedAppleTalkLink,              37).
-define(FramedAppleTalkNetwork,           38).
-define(FramedAppleTalkZone,              39).
-define(AcctStatusType,                   40).
-define(AcctDelayTime,                    41).
-define(AcctInputOctets,                  42).
-define(AcctOutputOctets,                 43).
-define(AcctSessionId,                    44).
-define(AcctAuthentic,                    45).
-define(AcctSessionTime,                  46).
-define(AcctInputPackets,                 47).
-define(AcctOutputPackets,                48).
-define(AcctTerminateCause,               49).
-define(AcctMultiSessionId,               50).
-define(AcctLinkCount,                    51).
-define(AcctInputGigawords,               52).
-define(AcctOutputGigawords,              53).
-define(EventTimestamp,                   55).
-define(EgressVLANID,                     56).
-define(IngressFilters,                   57).
-define(EgressVLANName,                   58).
-define(UserPriorityTable,                59).
-define(ChapChallenge,                    60).
-define(NasPortType,                      61).
-define(PortLimit,                        62).
-define(LoginLatPort,                     63).
-define(TunnelType,                       64).
-define(TunnelMediumType,                 65).
-define(TunnelClientEndpoint,             66).
-define(TunnelServerEndpoint,             67).
-define(AcctTunnelConnection,             68).
-define(TunnelPassword,                   69).
-define(ARAPPassword,                     70).
-define(ARAPFeatures,                     71).
-define(ARAPZoneAccess,                   72).
-define(ARAPSecurity,                     73).
-define(ARAPSecurityData,                 74).
-define(PasswordRetry,                    75).
-define(Prompt,                           76).
-define(ConnectInfo,                      77).
-define(ConfigurationToken,               78).
-define(EAPMessage,                       79).
-define(MessageAuthenticator,             80).
-define(TunnelPrivateGroupID,             81).
-define(TunnelAssignmentID,               82).
-define(TunnelPreference,                 83).
-define(ARAPChallengeResponse,            84).
-define(AcctInterimInterval,              85).
-define(AcctTunnelPacketsLost,            86).
-define(NasPortId,                        87).
-define(FramedPool,                       88).
-define(CUI,                              89).
-define(TunnelClientAuthID,               90).
-define(TunnelServerAuthID,               91).
-define(NasFilterRule,                    92).
-define(OriginatingLineInfo,              94).
-define(NasIPv6Address,                   95).
-define(FramedInterfaceId,                96).
-define(FramedIPv6Prefix,                 97).
-define(LoginIPv6Host,                    98).
-define(FramedIPv6Route,                  99).
-define(FramedIPv6Pool,                  100).
-define(ErrorCause,                      101).
-define(EAPKeyName,                      102).
-define(DigestResponse,                  103).
-define(DigestRealm,                     104).
-define(DigestNonce,                     105).
-define(DigestResponseAuth,              106).
-define(DigestNextnonce,                 107).
-define(DigestMethod,                    108).
-define(DigestURI,                       109).
-define(DigestQop,                       110).
-define(DigestAlgorithm,                 111).
-define(DigestEntityBodyHash,            112).
-define(DigestCNonce,                    113).
-define(DigestNonceCount,                114).
-define(DigestUsername,                  115).
-define(DigestOpaque,                    116).
-define(DigestAuthParam,                 117).
-define(DigestAKAAuts,                   118).
-define(DigestDomain,                    119).
-define(DigestStale,                     120).
-define(DigestHA1,                       121).
-define(SIPAOR,                          122).
-define(DelegatedIPv6Prefix,             123).
-define(MIP6FeatureVector,               124).
-define(MIP6HomeLinkPrefix,              125).
-define(OperatorName,                    126).
-define(LocationInformation,             127).
-define(LocationData,                    128).
-define(BasicLocationPolicyRules,        129).
-define(ExtendedLocationPolicyRules,     130).
-define(LocationCapable,                 131).
-define(RequestedLocationInfo,           132).
-define(FramedManagementProtocol,        133).
-define(ManagementTransportProtection,   134).
-define(ManagementPolicyId,              135).
-define(ManagementPrivilegeLevel,        136).
-define(PKMSSCert,                       137).
-define(PKMCACert,                       138).
-define(PKMConfigSettings,               139).
-define(PKMCryptosuiteList,              140).
-define(PKMSAID,                         141).
-define(PKMSADescriptor,                 142).
-define(PKMAuthKey,                      143).
-define(DSLiteTunnelName,                144).
-define(MobileNodeIdentifier,            145).
-define(ServiceSelection,                146).
-define(PMIP6HomeLMAIPv6Address,         147).
-define(PMIP6VisitedLMAIPv6Address,      148).
-define(PMIP6HomeLMAIPv4Address,         149).
-define(PMIP6VisitedLMAIPv4Address,      150).
-define(PMIP6HomeHNPrefix,               151).
-define(PMIP6VisitedHNPrefix,            152).
-define(PMIP6HomeInterfaceID,            153).
-define(PMIP6VisitedInterfaceID,         154).
-define(PMIP6HomeIPv4HoA,                155).
-define(PMIP6VisitedIPv4HoA,             156).
-define(PMIP6HomeDHCP4ServerAddress,     157).
-define(PMIP6VisitedDHCP4ServerAddress,  158).
-define(PMIP6HomeDHCP6ServerAddress,     159).
-define(PMIP6VisitedDHCP6ServerAddress,  160).
-define(PMIP6HomeIPv4Gateway,            161).
-define(PMIP6VisitedIPv4Gateway,         162).
-define(EAPLowerLayer,                   163).
-define(GSSAcceptorServiceName,          164).
-define(GSSAcceptorHostName,             165).
-define(GSSAcceptorServiceSpecifics,     166).
-define(GSSAcceptorRealmName,            167).
-define(FramedIPv6Address,               168).
-define(DNSServerIPv6Address,            169).
-define(RouteIPv6Information,            170).
-define(DelegatedIPv6PrefixPool,         171).
-define(StatefulIPv6AddressPool,         172).
-define(IPv66rdConfiguration,            173).
-define(AllowedCalledStationId,          174).
-define(EAPPeerId,                       175).
-define(EAPServerId,                     176).
-define(MobilityDomainId,                177).
-define(PreauthTimeout,                  178).
-define(NetworkIdName,                   179).
-define(EAPoLAnnouncement,               180).
-define(WLANHESSID,                      181).
-define(WLANVenueInfo,                   182).
-define(WLANVenueLanguage,               183).
-define(WLANVenueName,                   184).
-define(WLANReasonCode,                  185).
-define(WLANPairwiseCipher,              186).
-define(WLANGroupCipher,                 187).
-define(WLANAKMSuite,                    188).
-define(WLANGroupMgmtCipher,             189).
-define(WLANRFBand,                      190).

%% AcctStatusType attribute values
-define(AccountingStart,                   1).
-define(AccountingStop,                    2).
-define(AccountingInterimUpdate,           3).
-define(AccountingON,                      7).
-define(AccountingOFF,                     8).

%% ErrorCause attribute values
-define(ResidualSessionContextRemoved,   201).
-define(InvalidEapPacket,                202).
-define(UnsupportedAttribute,            401).
-define(MissingAttribute,                402).
-define(NasIdentificationMismatch,       403).
-define(InvalidRequest,                  404).
-define(UnsupportedService,              405).
-define(UnsupportedExtension,            406).
-define(AdministrativelyProhibited,      501).
-define(RequestNotRoutable,              502).
-define(SessionContextNotFound,          503).
-define(SessionContextNotRemovable,      504).
-define(OtherProxyProcessingError,       505).
-define(ResourcesUnavailable,            506).
-define(RequestInitiated,                507).

%% Vendor-Specific Private Enterprise Codes
-define(Cisco,                             9).
-define(Microsoft,                       311).
-define(Ascend,                          529).
-define(Mikrotik,                      14988).
-define('3GPP',                        10415).

%% Cisco Vendor-Specific (sub)attributes
-define(H323CallOrigin,                   26).

%% Microsoft Vendor-Specific (sub)attributes
-define(MsChapChallenge,                  11).
-define(MsChapResponse,                    1).
-define(MsChapDomain,                     10).
-define(MsChapError,                       2).
-define(MsChapCpw1,                        3).
-define(MsChapCpw2,                        4).
-define(MsChapLmEncPw,                     5).
-define(MsChapNtEncPw,                     6).
-define(MsChap2Response,                  25).
-define(MsChap2Success,                   26).
-define(MsChap2Cpw,                       27).
-define(MsChapMppeKeys,                   12).
-define(MsMppeSendKey,                    16).
-define(MsMppeRecvKey,                    17).
-define(MsMppeEncryptionPolicy,            7).
-define(MsMppeEncryptionTypes,             8).
-define(MsBapUsage,                       13).
-define(MsLinkUtilizationThreshold,       14).
-define(MsLinkDropTimeLimit,              15).
-define(MsOldArapPassword,                19).
-define(MsNewArapPassword,                20).
-define(MsArapPasswordChangeReason,       21).
-define(MsArapChallenge,                  33).
-define(MsRasVendor,                       9).
-define(MsRasVersion,                     18).
-define(MsFilter,                         22).
-define(MsAcctAuthType,                   23).
-define(MsAcctEapType,                    24).
-define(MsPrimaryDnsServer,               28).
-define(MsSecondaryDnsServer,             29).
-define(MsPrimaryNbnsServer,              30).
-define(MsSecondaryNbnsServer,            31).

%% Ascend Vendor-Specific (sub)attributes
-define(AscendClientGateway,             132).
-define(AscendDataRate,                  197).
-define(AscendXmitRate,                  255).

%% Mikrotik Vendor-Specific (sub)attributes
-define(MikrotikRecvLimit,                 1).
-define(MikrotikXmitLimit,                 2).
-define(MikrotikGroup,                     3).
-define(MikrotikWirelessForward,           4).
-define(MikrotikWirelessSkipDot1x,         5).
-define(MikrotikWirelessEncAlgo,           6).
-define(MikrotikWirelessEncKey,            7).
-define(MikrotikRateLimit,                 8).
-define(MikrotikRealm,                     9).
-define(MikrotikHostIp,                   10).
-define(MikrotikMarkId,                   11).
-define(MikrotikAdvertiseUrl,             12).
-define(MikrotikAdvertiseInterval,        13).
-define(MikrotikRecvLimitGigawords,       14).
-define(MikrotikXmitLimitGigawords,       15).
-define(MikrotikWirelessPsk,              16).
-define(MikrotikTotalLimit,               17).
-define(MikrotikTotalLimitGigawords,      18).
-define(MikrotikAddressList,              19).
-define(MikrotikWirelessMpKey,            20).
-define(MikrotikWirelessComment,          21).
-define(MikrotikDelegatedIpv6Pool,        22).
-define(MikrotikDhcpOptionSet,            23).
-define(MikrotikDhcpOptionParamStr1,      24).
-define(MikrotikDhcpOptionParamStr2,      25).
-define(MikrotikWirelessVlanId,           26).
-define(MikrotikWirelessVlanIdType,       27).
-define(MikrotikWirelessMinSignal,        28).
-define(MikrotikWirelessMaxSignal,        29).

%% 3GPP Vendor-Specific (sub)attributes
-define('3GPP-IMSI',                       1).
-define('3GPP-Charging-ID',                2).
-define('3GPP-PDP-Type',                   3).
-define('3GPP-RAT-Type',                  21).
-define('TWAN-Identifier',                29).

